/********************************************************

  file xbcutstk.c
  ```````````````
  Cutting stock problem, solved by column (= cutting
  pattern) generation heuristic looping over the root
  node.

  (c) 2001 Dash Associates
      author: S.Heipcke
              Peter Notebaert
********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <crtdbg.h>

#include "lp_lib.h"

#define EPS 1e-6
#define MAXCOL 1000

#if !defined FALSE
# define FALSE 0
#endif

#if !defined TRUE
# define TRUE 1
#endif

#if !defined MIN
# define MIN(a, b) ((a) < (b) ? a : b)
#endif

/****DATA****/
int NWIDTHS;
double MAXWIDTH, CUTWASTE;

double *WIDTH;  /* Possible widths */
int *DEMAND;    /* Demand per width */

double knapsack(int N, double *c, double *a, double R, int *xbest);

/***********************************************************************/

void print_pattern(lprec *prob)
{
 int i, j, Ncolumn;
 REAL *col;

 Ncolumn=get_Ncolumns(prob);
 col=(REAL *) malloc((1+NWIDTHS)*sizeof(*col));
 for (i=1;i<=Ncolumn;i++) {
  get_column(prob, i, col);
  printf("pat_%d:", i);
  for (j=1;j<=NWIDTHS;j++)
   if (col[j]>EPS)
    printf(" %g:%.0f", WIDTH[j]-CUTWASTE, col[j]);
  printf("\n");
 }
 free(col);
}

/* generate a starting model */

/* This one is an easy one. Each pattern is just the maximum number of cuts out of width */
/* There are better algorithms for a starting pattern which result in less columns needed to be generated afterwards. */

lprec *modcutstock_1cut()
{
 lprec *prob;
 int i,j;
 REAL *row;
 int *PATTERNS;  /* (Basic) cutting patterns */

 printf("1 cut starting model\n");

 prob = make_lp(0, NWIDTHS);           /* Initialize a new lp problem */
 set_verbose(prob, 0);

 PATTERNS = (int *) malloc(NWIDTHS*sizeof(*PATTERNS));

 for(j=0;j<NWIDTHS;j++) PATTERNS[j]=(int)floor(MAXWIDTH/WIDTH[1+j]);

/****VARIABLES****/
 for(j=0;j<NWIDTHS;j++) {
  char name[10];

  sprintf(name, "pat_%d",j+1);
  set_col_name(prob, j+1, name);
  set_bounds(prob, j+1, 0, (int)ceil((double)DEMAND[1+j]/PATTERNS[j]));
 }

 row = malloc((1+NWIDTHS)*sizeof(*row));
 row[0]=0;

/****OBJECTIVE****/
 for(j=1;j<=NWIDTHS;j++)
  row[j]=1.0;
 set_obj_fn(prob, row);

/****CONSTRAINTS****/
 for(i=0;i<NWIDTHS;i++)
 {
  for(j=0;j<NWIDTHS;j++)
   row[j+1]=((i==j) ? PATTERNS[j] : 0);
  add_constraint(prob, row, GE, DEMAND[1+i]);
 }

 print_pattern(prob);
 printf("\n");

 //write_lp(prob, "con");

 free(row);
 free(PATTERNS);

 return(prob);
}

int *sorter;

int compareasc(const int *a, const int *b)
{
    //int i1=a-sorter, i2=b-sorter;
	int i1=*a, i2=*b;
    return(WIDTH[1+i1] < WIDTH[1+i2] ? -1 : WIDTH[1+i1] > WIDTH[1+i2] ? +1 : 0);
}

int comparedesc(const int *a, const int *b)
{
    //int i1=a-sorter, i2=b-sorter;
	int i1=*a, i2=*b;
    return(WIDTH[1+i1] < WIDTH[1+i2] ? +1 : WIDTH[1+i1] > WIDTH[1+i2] ? -1 : 0);
}

/* sort WIDTH and DEMAND according to WIDTHS ascending */
void sort(int compare(const int *, const int *))
{
 int j;
 double *WIDTH0;
 int *DEMAND0;

 sorter=(int *) malloc(NWIDTHS*sizeof(*sorter));
 for (j=0;j<NWIDTHS;j++)
  sorter[j]=j;
 qsort(sorter, NWIDTHS, sizeof(*sorter), compare);
 WIDTH0=WIDTH;
 DEMAND0=DEMAND;
 WIDTH=(double *) malloc((1+NWIDTHS)*sizeof(*WIDTH0));
 DEMAND=(int *) malloc((1+NWIDTHS)*sizeof(*DEMAND0));
 for (j=0;j<NWIDTHS;j++) {
  WIDTH[1+j]=WIDTH0[1+sorter[j]];
  DEMAND[1+j]=DEMAND0[1+sorter[j]];
 }
 free(DEMAND0);
 free(WIDTH0);
 free(sorter);
}

/* starting model from vasek chvatal (finding a good initial solution) */
/* This is a very fast procedure for finding a reasonable good initial solution */
/* If the initial value of the objective function is close to the optimum value, then the number of simplex
   iterations required to reach the optimum is likely to be small.
   Remark that each iteration results in the generation of a column.
   So it is very benifitial to find an initial solution that will result in less iterations */
/* This procedure is based on the assumption that it pays to get the wide finals out of the way first, letting the versatile narrow ones
   complete for the leftovers */

lprec *modcutstock_chvatal()
{
 lprec *prob;
 char *R;
 int *b;
 double r, a, ratio, dw;
 REAL *col;
 int i, j, k;
 char name[10];

 printf("chvatal starting model\n");

 prob = make_lp(NWIDTHS, 0);           /* Initialize a new lp problem */
 set_verbose(prob, 0);

 sort(comparedesc);

 R=(char *) malloc(NWIDTHS*sizeof(*R));
 memset(R, TRUE, NWIDTHS*sizeof(*R));
 b=(int *) malloc(NWIDTHS*sizeof(*b));
 memcpy(b, DEMAND+1, NWIDTHS*sizeof(*b));

 col = (double *) malloc((1+NWIDTHS)*sizeof(*col));
 col[0]=1; /****OBJECTIVE****/

 for (j=0;j<NWIDTHS;j++) {
  r=MAXWIDTH;
  ratio=0;
  dw=0;
  k=-1;
  for (i=0;i<NWIDTHS;i++) {
   if (R[i]) {
    col[1+i]=floor(r/WIDTH[1+i]);
    if (col[1+i]>EPS) {
     r-=col[1+i]*WIDTH[1+i];
     a=b[i]/col[1+i];
     if ((k<0) || (a<ratio)) {
      ratio=a;
      k=i;
     }
     if((int)ceil((double)DEMAND[1+i]/col[1+i]) > dw)
      dw = (int)ceil((double)DEMAND[1+i]/col[1+i]);
    }
   }
   else {
    col[1+i]=0;
   }
  }
  R[k]=FALSE;
  for (i=0;i<NWIDTHS;i++)
   b[i]-=(int)(ratio*col[1+i]+EPS);
  add_column(prob, col);
  set_upbo(prob, j+1, dw);
  sprintf(name, "pat_%d",j+1);
  set_col_name(prob, j+1, name);

  set_constr_type(prob, j+1, GE);
  set_rh(prob, j+1, DEMAND[1+j]);
 }

 print_pattern(prob);
 printf("\n");

 //write_lp(prob, "con");

 free(col);
 free(b);
 free(R);

 return(prob);
}

/* first fit decreasing model generator */

lprec *modcutstock_firstfitdecreasing()
{
 lprec *prob;
 int *b;
 double r, a, ratio, dw;
 REAL *col;
 int i, j, k;
 char name[10];

 printf("first fit decreasing starting model\n");

 prob = make_lp(NWIDTHS, 0);           /* Initialize a new lp problem */
 set_verbose(prob, 0);

 sort(comparedesc);

 b=(int *) malloc(NWIDTHS*sizeof(*b));
 memcpy(b, DEMAND+1, NWIDTHS*sizeof(*b));

 col = (double *) malloc((1+NWIDTHS)*sizeof(*col));
 col[0]=1; /****OBJECTIVE****/

 j=0;
 do {
  r=MAXWIDTH;
  ratio=0;
  dw=0;
  k=-1;
  for (i=0;i<NWIDTHS;i++) {
   col[1+i]=floor(r/WIDTH[1+i]);
   if (b[i]<col[1+i])
    col[1+i]=b[i];
   if (col[1+i]>EPS) {
    r-=col[1+i]*WIDTH[1+i];
    a=b[i]/col[1+i];
    if ((k<0) || (a<ratio)) {
     ratio=a;
     k=i;
    }
    if((int)ceil((double)DEMAND[1+i]/col[1+i]) > dw)
     dw = (int)ceil((double)DEMAND[1+i]/col[1+i]);
   }
  }
  ratio=floor(ratio+EPS);
  for (k=-1,i=0;i<NWIDTHS;i++) {
   b[i]-=(int)(ratio*col[1+i]+EPS);
   if (b[i]>EPS)
    k=i;
  }
  add_column(prob, col);
  set_upbo(prob, ++j, dw);
  sprintf(name, "pat_%d",j);
  set_col_name(prob, j, name);
 } while (k!=-1);

 for (i=0;i<NWIDTHS;i++) {
  set_constr_type(prob, i+1, GE);
  set_rh(prob, i+1, DEMAND[1+i]);
 }

 print_pattern(prob);
 printf("\n");

 //write_lp(prob, "con");

 free(col);
 free(b);

 return(prob);
}

int genint(int mlow, int mhigh)
{
 int t;
 static int peter=0;

 //t = rand()*(mhigh-mlow)/RAND_MAX + mlow;
 peter++;
 t=(int) (fmod(peter*99, 43) / 42 * (mhigh+.99-mlow) + mlow);
 return(t);
}

int improve(double **prod, int n, int *numsize, double swidth)
{
 int i, j, r, change, pi, pj;
 double s;
 char junk;
 double *sizeu, maxvac, maxwidth, tmp;

 sizeu = (double *) malloc(n * sizeof(*sizeu));
 for (i = 0; i < n; i++) {
  for (s = 0, j = 0; j < numsize[i]; j++)
   s += prod[i][j];
  sizeu[i] = s;
 }

 /* -- Improve Value */
 change = 1;
 junk = FALSE;
 while (change <= 10) {
  junk = TRUE;
  switch (change) {
  case 1:
   maxwidth = swidth*0.9;
   break;
  case 2:
   maxwidth = swidth*0.94;
   break;
  case 3:
   maxwidth = swidth*0.96;
   break;
  case 4:
   maxwidth = swidth*0.98;
   break;
  default:
   maxwidth = swidth*0.99;
   break;
  }
  for (i = 0; i < n-1; i++) {
   if (sizeu[i] > maxwidth)
    continue;
   for (j = 1; j < n; j++) {
	maxvac = swidth - MIN(sizeu[i], sizeu[j]);
	pi = 0;
	while (pi < numsize[i]) {
	 pj = 0;
	 while (pj < numsize[j]) {
	  /* -- See if swapping is possible */
	  if ((-prod[i][pi]+prod[j][pj]+sizeu[i] <= swidth) && (-prod[j][pj]+prod[i][pi]+sizeu[j] <= swidth)) {
	   /* -- See if swapping increases maxvacancy */
	   if (swidth - MIN(-prod[i][pi]+prod[j][pj]+sizeu[i], -prod[j][pj]+prod[i][pi]+sizeu[j]) > maxvac) {
		/* --Do the swapping */
		sizeu[i] += prod[j][pj] - prod[i][pi];
		sizeu[j] += prod[i][pi] - prod[j][pj];
	    tmp = prod[i][pi];
		prod[i][pi] = prod[j][pj];
		prod[j][pj] = tmp;
		maxvac = swidth - MIN(sizeu[i], sizeu[j]);
		junk = FALSE;
	   }
	  }
	  /* -- See if inserting pi into prod[j] is possible */
	  if (sizeu[j]+prod[i][pi] <= swidth) {
	   /* -- See if inserting increases maxvac */
	   if (swidth - min(sizeu[j]+prod[i][pi], sizeu[i]-prod[i][pi]) > maxvac) {
		/* -- Do the insertion */
		sizeu[i] -= prod[i][pi];
		sizeu[j] += prod[i][pi];
		numsize[j]++;
		prod[j] = (double *) realloc(prod[j], numsize[j] * sizeof(**prod));
		prod[j][numsize[j]-1] = prod[i][pi];
		numsize[i]--;
		memcpy(prod[i] + pi, prod[i] + pi + 1, (numsize[i] - pi) * sizeof(**prod));
		prod[i][numsize[i]] = 0;
		maxvac = swidth - MIN(sizeu[i], sizeu[j]);
		junk = FALSE;
	   }
	  }
	  pj++;
	 }
	 pi++;
	}
   }
  }
  change++;
  if (junk)
   break;
 }

 r = 0;
 for (i = 0; i < n; i++) {
  if (numsize[i] > 0)
   r++;
 }
 return(r);
}

int comparedoubleasc(const double *a, const double *b)
{
    return(*a < *b ? -1 : *a > *b ? +1 : 0);
}

void dupprod(int n, double **prod, int *numsize, double ***proddup, int **numsizedup)
{
 double **prod2;
 int *numsize2;
 int i;

 numsize2 = (int *) malloc(n * sizeof(*numsize2));
 memcpy(numsize2, numsize, n * sizeof(*numsize2));

 prod2 = (double **) malloc(n * sizeof(*prod2));
 for (i = 0; i < n; i++) {
  prod2[i] = (double *) malloc(numsize2[i] * sizeof(**prod2));
  memcpy(prod2[i], prod[i], numsize2[i] * sizeof(**prod2));
 }
 *proddup = prod2;
 *numsizedup = numsize2;
}

void freeprod(int n, double **prod, int *numsize, int *count)
{
 int i;

 if (prod != NULL) {
  for (i = 0; i < n; i++)
   if (prod[i] != NULL)
    free(prod[i]);
  free(prod);
 }
 if (numsize != NULL)
  free(numsize);
 if (count != NULL)
  free(count);
}

void printprod(int n, double **prod, int *numsize, int *count)
{
 int i, j, x;
 double a;

 x = 0;
 for (i = 0; i < n; i++)
  if (count[i]) {
   a = 0;
   printf("%5d. ", ++x);
   for (j = 0; j < numsize[i]; j++) {
    printf("%f ", prod[i][j]);
    a += prod[i][j];
   }
   printf("#%f * %d\n", a, count[i]);
  }
}

int sortx(int n, double **prod, int *numsize, int **pcount)
{
 int i, j, k, x;
 int *count;
 double *prodi, *prodj;

 count = (int *) malloc(n * sizeof(*count));
 for (i = 0; i < n; i++) {
  qsort(prod[i], numsize[i], sizeof(**prod), comparedoubleasc);
  count[i] = 1;
 }

 x = 0;
 for (i = 0; i < n; i++) {
  if (count[i]) {
   if (numsize[i]) {
    x++;
    for (j = i + 1; j < n; j++) {
     if ((count[j]) && (numsize[j] == numsize[i])) {
	  prodi = prod[i];
	  prodj = prod[j];
	  for (k = numsize[j] - 1; (k >= 0) && (prodi[k] == prodj[k]) ; k--);
	  if (k < 0) {
	   count[i] += count[j];
	   count[j] = 0;
	   free(prod[j]);
	   prod[j] = NULL;
	  }
	 }
	}
   }
   else {
    count[i] = 0;
	free(prod[i]);
	prod[i] = NULL;
   }
  }
 }
 *pcount = count;
 return(x);
}

void addpp(double ***prod, int *n, int **numsize, double zz[], int r)
{
 double *pprod;
#define step 100

 if (*n == 0) {
  *prod = (double **) malloc(step * sizeof(***prod));
  *numsize = (int *) malloc(step * sizeof(**numsize));
 }
 else if (((*n) % step) == 0) {
  *prod = (double **) realloc(*prod, (*n + step) * sizeof(***prod));
  *numsize = (int *) realloc(*numsize, (*n + step) * sizeof(**numsize));
 }
 (*prod)[*n] = pprod = (double *) malloc(r * sizeof(*pprod));
 memcpy(pprod, zz, r * sizeof(*pprod));
 (*numsize)[*n] = r;
 (*n)++;
#undef step
}

lprec *modcutstock_Patodia()
{
 lprec *prob;
 double *col;
 int k;
 double d;

 int noiter;
 int i, j, q;
 double ndemand, maxj;
 char name[10];
 int minval, mtimes;
 int it;
 double *WIDTH1;
 int *DEMAND1;
 double t, minx, maxx;
 int a;
 double bal;
 double zz[100];
 int r;
 double tmin;
 int el;
 int n, n0 = 0;
 double **prod, **prod0 = NULL;
 int *numsize = NULL, *numsize0 = NULL, *count0 = NULL;

#define xiter 5
#define rlwidth MAXWIDTH

 printf("Patodia starting model\n");

 noiter = xiter;

 sort(compareasc);

 ndemand = 0;
 for (i = 1; i <= NWIDTHS; i++)
  ndemand += WIDTH[i]*DEMAND[i];

 maxj = ndemand / rlwidth;

 srand(-5);

 //clear
 minval = 10000;
 mtimes = 10000;

 WIDTH1=(double *) malloc((1+NWIDTHS)*sizeof(*WIDTH1));
 DEMAND1=(int *) malloc((1+NWIDTHS)*sizeof(*DEMAND1));

 for (it = 1; it <= noiter; it++) {
  memcpy(WIDTH1+1, WIDTH+1, NWIDTHS*sizeof(*WIDTH1));
  memcpy(DEMAND1+1, DEMAND+1, NWIDTHS*sizeof(*DEMAND1));

  j = 0;
  q = NWIDTHS;

  n = 0;
  r = 0;
  while (TRUE) {
   t = 0;
   minx = WIDTH1[1];
   maxx = WIDTH1[q];

   if (q > 1) {
	if (DEMAND1[1] == 0)
	 minx = WIDTH1[2];
	if (DEMAND1[q] == 0)
	 maxx = WIDTH1[q-1];
   }
   while (TRUE) {
	a = genint(1, q);
	if (DEMAND1[a] > 0) {
	 if (t <= rlwidth - WIDTH1[a]) {
	  t += WIDTH1[a];
	  DEMAND1[a]--;
	  if ((DEMAND1[a] == 0) && (q > 1)) {
	   if (a == 1)
		minx = WIDTH1[2];
	   if (a == q)
		maxx = WIDTH1[q-1];
	  }
	  zz[r++] = WIDTH1[a];
	 }
	}
	else {
	 if ((q == 1) && (DEMAND1[q] == 0)) {
	  addpp(&prod, &n, &numsize, zz, r);
	  q = q - 1;
	  break;
	 }
	 else if ((q == 2) && (DEMAND1[1] == 0) && (DEMAND1[2] == 0)) {
      addpp(&prod, &n, &numsize, zz, r);
	  q = q - 2;
	  break;
	 }
	 else if ((q == 3) && (DEMAND1[1] == 0) && (DEMAND1[2] == 0) && (DEMAND1[3] == 0)) {
      addpp(&prod, &n, &numsize, zz, r);
	  q = q - 3;
	  break;
	 }
	 else
	  continue;
	}
	bal = rlwidth - t;
	/* Cannot allocate anything now, so exit */
	if (bal < minx) {
	 /* -- backtrack */
     qsort(zz, r, sizeof(*zz), comparedoubleasc);
	 tmin = zz[0];
	 t -= tmin;
	 bal = rlwidth - t;
	 for (el = 0, i = 1; (i <= q) && (WIDTH1[i] != tmin); i++);
	 if (i <= q)
	  a = i;
	 else
	  a = 0;
	 DEMAND1[a]++;
	 if (r > 1)
	  memcpy(zz, zz + 1, (--r) * sizeof(*zz));
	 else
	  r = 0;
	 for (a = q; a >= 1; a--) {
	  if ((bal >= WIDTH1[a]) && (DEMAND1[a] > 0)) {
	   t += WIDTH1[a];
	   DEMAND1[a]--;
	   zz[r++] = WIDTH1[a];
	   break;
	  }
	 }
	 /* -- At end add bstr to cstr and then following line */
	 addpp(&prod, &n, &numsize, zz, r);
	 r = 0;
	 /* -- Remove array which are not required */
	 a = 1;
	 while (a <= q) {
	  if (DEMAND1[a] == 0) {
	   memcpy(WIDTH1+a, WIDTH1+a+1, (q-a)*sizeof(*WIDTH1));
	   memcpy(DEMAND1+a, DEMAND1+a+1, (q-a)*sizeof(*DEMAND1));
	   q--;
	   if (q == 0)
	    break;
	   continue;
	  }
	  a++;
	 }
	 break;
	}
	/* -- Assign biggest possible and exit */
	if ((bal >= minx) && (bal <= maxx)) {
	 for (a = q; a >= 1; a--) {
	  if (bal >= WIDTH1[a] && (DEMAND1[a] > 0)) {
	   t += WIDTH1[a];
	   DEMAND1[a]--;
	   zz[r++] = WIDTH1[a];
	   break;
	  }
	 }
	 if (q == 0)
	  break;
	 /* -- At end add bstr to cstr and then following line */
	 addpp(&prod, &n, &numsize, zz, r);
	 r = 0;
	 /* -- Remove array which are not required */
	 a = 1;
	 while (a <= q) {
	  if (DEMAND1[a] == 0) {
	   memcpy(WIDTH1+a, WIDTH1+a+1, (q-a)*sizeof(*WIDTH1));
	   memcpy(DEMAND1+a, DEMAND1+a+1, (q-a)*sizeof(*DEMAND1));
	   q--;
	   if (q == 0)
	    break;
	   continue;
	  }
	  a++;
	 }
	 break;
	}
   }
   j++;
   if (q == 0)
    break;
  }

  r = improve(prod, n, numsize, rlwidth);
  
  printf("Rols: %d\n", r);

  j = r;
  if (j <= minval) {
   double **proddup;
   int rx, *numsizedup, *count;

   dupprod(n, prod, numsize, &proddup, &numsizedup);
   rx = sortx(n, proddup, numsizedup, &count);
   if (rx < mtimes) {
    freeprod(n0, prod0, numsize0, count0);
	n0 = n;
    prod0 = proddup;
	numsize0 = numsizedup;
	count0 = count;
	mtimes = rx;
	minval = j;
	printf("Improved: %d %d\n", minval, rx);
   }
   else {
    freeprod(n, proddup, numsizedup, NULL);
   }
  }
 }

 freeprod(n, prod, numsize, NULL);

 //printprod(n0, prod0, numsize0, count0);

 prob = make_lp(NWIDTHS, 0);           /* Initialize a new lp problem */
 set_verbose(prob, 0);

 col = (double *) malloc((1+NWIDTHS)*sizeof(*col));
 col[0]=1; /****OBJECTIVE****/

 n=0;
 for (j = 0; j < n0; j++) {
  if (count0[j]) {
   memset(col + 1, 0, NWIDTHS * sizeof(*col));
   for (i = 0; i < numsize0[j]; i++) {
    d = prod0[j][i];
    for (k = 1; (k <= NWIDTHS) && (WIDTH[k] != d); k++);
    if (k <= NWIDTHS)
     col[k]++;
   }
   add_column(prob, col);
   //set_upbo(prob, ++j, dw);
   n++;
   sprintf(name, "pat_%d",n);
   set_col_name(prob, n, name);
  }
 }
 free(col);

 for (i=0;i<NWIDTHS;i++) {
  set_constr_type(prob, i+1, GE);
  set_rh(prob, i+1, DEMAND[1+i]);
 }

 print_pattern(prob);
 printf("\n");

 //set_obj_bound(prob, minval);
 set_break_at_value(prob, minval+EPS);

 //write_lp(prob, "con");

 return(prob);
}

/**************************************************************************/
/*  Column generation loop at the top node:                               */
/*    solve the LP                                                        */
/*    get the solution values                                             */
/*    generate new column(s) (=cutting pattern)                           */
/*    load the modified problem                                           */
/**************************************************************************/
void solvecutstock(lprec *prob, char firstsolution)
{
 double objval;                  /* Objective value */
 double objval0;
 int i,j;
 int npatt, npass;               /* Counters for columns and passes */
 double *soluse;                 /* Solution values for variables pat */
 double *column;
 int *rowno;
 double *dualdem;                /* Dual values of demand constraints */
 double dw,z;
 int *x;
 char name[10];
 int ret;
 int len=0;

 if (!firstsolution)
  set_break_at_value(prob, get_infinite(prob));

 npatt = get_Ncolumns(prob);

 x=(int *) malloc(NWIDTHS*sizeof(*x));
 column=(REAL *) malloc((1+NWIDTHS)*sizeof(*column));
 rowno=(int *) malloc((1+NWIDTHS)*sizeof(*rowno));

 for(npass=0;npass<MAXCOL;npass++)
 {
  //write_lp(prob, "con");

  ret=solve(prob);               /* Solve the LP */
  _ASSERT(ret==0);

  objval = get_objective(prob);  /* Get the objective value */

  len=printf("suboptimal solution: %g, time: %.0f\n", objval, time_elapsed(prob));

  /* Get the solution values: */
  get_ptr_variables(prob, &soluse);

  /*
  printf("Optimal solution: %g rolls, %d patterns\n", objval, npatt);
  printf("Rolls per pattern:\n");
  for(i=0;i<npatt;i++) printf("pat_%d: %g\n", i+1, soluse[i]);
  printf("\n");
  */

  get_ptr_dual_solution(prob, &dualdem);

      /* Solve integer knapsack problem  z = min{cx : ax<=r, x in Z^n}
         with r=MAXWIDTH, n=NWIDTHS */
  if (!firstsolution)
   z = knapsack(NWIDTHS, dualdem, WIDTH, MAXWIDTH, x);

  if((firstsolution) || (z < 1+EPS))
  {
   /*
   if (z < 1+EPS)
    firstsolution = FALSE;
   */
   //printf("\n"); write_lp(prob, "con");

   //printf("no profitable column found.\n\n");
   printf("\n%srelaxed solution = %g.", (firstsolution) ? "" : "optimal ", objval);
   if (!firstsolution)
    printf(" marginal cost = %g", z-1);

   objval0 = ceil(objval-EPS);

   printf("\nSolving for integer...\n");

   for(j=1;j<=npatt;j++)
    set_int(prob, j, TRUE);

   /* write_lp(prob, "xbcutstk.lp"); */

   set_verbose(prob, 4);
   ret=solve(prob);                    /* Solve the MIP */
   _ASSERT(ret==0 || ret==1);

   printf("time: %.f\n", time_elapsed(prob));

   /*
   get_ptr_dual_solution(prob, &dualdem);
   z = knapsack(NWIDTHS, dualdem, WIDTH, MAXWIDTH, x);
   */

   //if(z < 1+EPS)
    break;
   for(j=1;j<=npatt;j++)
    set_int(prob, j, FALSE);
  }

  npatt++;
  /* Print the new pattern: */
  //printf("new pattern found with marginal cost %g\n", z-1);
  printf("\r");
  len-=printf("pat_%d:", npatt);
  dw=0;
  for(i=0;i<NWIDTHS;i++)
  {
   if (x[i] != 0)
    len-=printf(" %g:%d", WIDTH[1+i]-CUTWASTE, x[i]);
   dw += WIDTH[1+i]*x[i];
  }
  if (len<0) len=0;
  printf("%*.*s\n", len, len, "");

  //printf("Total width: %g\n", dw);

  /* Create a new variable for this pattern: */
  j=0;
  rowno[j]=0; /* the objective */
  column[j++]=1;

  dw=0;
  for(i=0; i<NWIDTHS; i++)
   if(x[i] > EPS)
   {
    rowno[j]=1+i;
    column[j++]=x[i];
    if((int)ceil((double)DEMAND[1+i]/x[i]) > dw)
     dw = (int)ceil((double)DEMAND[1+i]/x[i]);
   }
  add_columnex(prob, j, column, rowno);

  sprintf(name, "pat_%d", npatt);
  set_col_name(prob, npatt, name);
  set_upbo(prob, npatt, dw);        /* Change the upper bound on the new var.*/
 }

 objval=get_objective(prob);
 get_ptr_variables(prob, &soluse);
 for(j=0,i=0;i<npatt;i++)
  if (soluse[i] != 0)
   j++;
 printf("%ssolution: %g rolls, %d patterns, %d used, %d passes\n", (firstsolution) ? "" : "optimal ", objval, npatt, j, npass);

 /* The delayed column generation algorithm is producing the optimal solution for the real (relaxed) model (model without integer constraints).
    However, since we need integer amounts, the final model is solved with all integer restrictions
    and this can result in a solution that is non-optimal because not all columns are available.
	(the delayed column generation algorithm only generates columns that are needed at each next iteration. This is fine of the relaxed model
	but if the integer model needs extra columns then this algorithm will not find the most optimal solution)
    If objective value of the integer model equals to the ceiling value of the relaxed solution (the best possible integer solution)
    then the found solution must be optimal.
    If the objective value of the integer model is greater than the ceiling value of the relaxed solution (the best possible integer solution)
    then it is not sure that the best solution is found. It maybe, it maybe not. One cannot know.
 */
 if ((objval - objval0 > EPS) || (firstsolution))
  printf("This may not be the most optimal solution\n");
 else
  printf("This should be the most optimal solution\n");

 printf("Rolls per used pattern:\n");
 for(i=0;i<npatt;i++)
  if (soluse[i] != 0)
   printf("pat_%d: %g\n", i+1, soluse[i]);
 printf("\n");

 free(rowno);
 free(column);
 free(x);
}

/**************************************************************************/
/* Integer Knapsack Algorithm for solving the integer knapsack problem    */
/*    z = max{cx : ax <= R, x in Z^N}                                     */
/* where there is an unlimited number of each type of item available.     */
/*                                                                        */
/* Input data:                                                            */
/*   N:        Number of item types                                       */
/*   c[i]:     Unit profit of item type i, i=1..n                         */
/*   a[i]:     Unit resource use of item type i, i=1..n                   */
/*   R:        Total resource available                                   */
/* Return values:                                                         */
/*   xbest[i]: Number of items of type i used in optimal solution         */
/*   zbest:    Value of optimal solution                                  */
/*                                                                        */
/* The algorithm assums that item types are ordered as follows            */
/*   c[0]/a[0] >= c[1]/a[1] >= ... >= c[N]/a[N]                           */
/**************************************************************************/
double knapsack(int N, double *c, double *a, double R, int *xbest)
{
 int j;
 double zbest = 0.0;
 lprec *kprob;
 int ret;
 REAL *x;

 /*
 printf ("Solving z = max{cx : ax <= b; x in Z^n}\n   c   =");
 for (j = 0; j < N; j++) printf (" %g,", c[j]);
 printf("\n   a   =");
 for (j = 0; j < N; j++) printf (" %g,", a[j]);
 printf("\n   c/a =");
 for (j = 0; j < N; j++) printf (" %g,", c[j]/a[j]);
 printf("\n   b   = %f\n", R);
 */

 kprob=make_lp(0, N);
 set_verbose(kprob, 0);
 for(j=1;j<=N;j++)
  set_int(kprob, j, TRUE);

 set_obj_fn(kprob, c);
 add_constraint(kprob, a, LE, R);

 set_maxim(kprob);
 //write_lp(kprob,"con");
 ret=solve(kprob);
 _ASSERT(ret==0);

 printf("knapsack solving time: %.f\n", time_elapsed(kprob));

 /*
 if (time_elapsed(kprob) > 2)
  write_lp(kprob, "xbcutstk.lp");
 */

 zbest = get_objective(kprob);

 get_ptr_variables(kprob, &x);
 for(j=0;j<N;j++) xbest[j]=(int)floor(x[j]+0.5);

 /*
 printf ("   z = %f\n   x =", zbest);
 for(j=0; j<N; j++) printf (" %d,", xbest[j]);
 printf("\n");
 */

 delete_lp(kprob);

 return (zbest);
}

/***********************************************************************/

char readinput(char *name)
{
    FILE *fp;

    WIDTH = NULL;
    DEMAND = NULL;
    NWIDTHS = 0;
    MAXWIDTH = 0;
    CUTWASTE = 0;

    fp = fopen(name, "r");
    if (fp != NULL) {
        char buf[256];
        int line = 0, i = 0;
#       define step 100

        while ((!feof(fp)) && (fgets(buf, sizeof(buf), fp) != NULL)) {
            line++;
            if (line == 1)
                MAXWIDTH = atof(buf);
            else if (line == 2)
                CUTWASTE = atof(buf);
            else if ((line % 2) == 1) {
				if (i == 0) {
                    WIDTH = (double *) malloc((1+step) * sizeof(*WIDTH));
					*WIDTH=0;
				}
                else if ((i % step) == 0)
                    WIDTH = (double *) realloc(WIDTH, (i + 1 + step) * sizeof(*WIDTH));
                WIDTH[1+i] = atof(buf);
            }
            else {
				if (i == 0) {
                    DEMAND = (int *) malloc((1+step) * sizeof(*DEMAND));
					*DEMAND=0;
				}
                else if ((i % step) == 0)
                    DEMAND = (int *) realloc(DEMAND, (i + 1 + step) * sizeof(*DEMAND));
                DEMAND[++i] = atoi(buf);
            }
        }
        NWIDTHS = i;
        fclose(fp);
        return((WIDTH != NULL) && (DEMAND != NULL) && (MAXWIDTH != 0) && (NWIDTHS != 0));
#       undef step
    }
    return(FALSE);
}

/* cutwaste is as if each cut is cutwaste larger than its actual width. 
   since there are n-1 cuts needed for n pieces or there is no cutwaste in the leftover of the last piece
   it is as if maxwidth is also cutwaste larger than its actual width */
void applycutwaste()
{
 int i;

 MAXWIDTH+=CUTWASTE;
 for (i=1; i<=NWIDTHS; i++)
  WIDTH[i]+=CUTWASTE;
}

void Usage()
{
 printf("Usage: xbcutstk inputfile [/f] [/c] [/p] [/1]\n");
 printf(" /o: 1 cut (default)\n");
 printf(" /f: first fit decreasing\n");
 printf(" /c: chvatal\n");
 printf(" /p: Patodia\n");
 printf("\n");
 printf(" /1: Stop after one iteration such that starting model is printed\n");
}

int main(int argc, char **argv)
{
 lprec *prob;
 int i;
 char firstsolution = FALSE;

 WIDTH=NULL;
 DEMAND=NULL;

 if (argc >= 2) {
  if (readinput(argv[1])) {
   applycutwaste();

   prob=NULL;
   for (i = 2; i < argc; i++) {
    if (strcmp(argv[i], "/o") == 0) {
	 if (prob != NULL) {
	  printf("%s argument may not be combined with other starting model generator options", argv[i]);
	  return(1);
	 }
	 prob=modcutstock_1cut();
	}
    else if (strcmp(argv[i], "/f") == 0) {
	 if (prob != NULL) {
	  printf("%s argument may not be combined with other starting model generator options", argv[i]);
	  return(1);
	 }
	 prob=modcutstock_firstfitdecreasing();
	}
	else if (strcmp(argv[i], "/c") == 0) {
	 if (prob != NULL) {
	  printf("%s argument may not be combined with other starting model generator options", argv[i]);
	  return(1);
	 }
	 prob=modcutstock_chvatal();
	}
	else if (strcmp(argv[i], "/p") == 0) {
	 if (prob != NULL) {
	  printf("%s argument may not be combined with other starting model generator options", argv[i]);
	  return(1);
	 }
	 prob=modcutstock_Patodia();
	}
	else if (strcmp(argv[i], "/1") == 0)
	 firstsolution = TRUE;
   }

   if (prob == NULL)
    prob=modcutstock_1cut();

   /* Solve the problem */
   solvecutstock(prob, firstsolution);

   delete_lp(prob);
  }
 }
 else
  Usage();

 if (DEMAND!=NULL)
  free(DEMAND);
 if (WIDTH!=NULL)
  free(WIDTH);
 return 0;
}